package eu.glorya.math.diplomka;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import eu.glorya.math.diplomka.gui.MenuBar;
import eu.glorya.math.diplomka.gui.PromotionPanel;
import eu.glorya.math.diplomka.gui.util.LookNFeel;

public class FactorTrees extends JFrame {
	private static final long serialVersionUID = -3419552514721122893L;
	private static final Log LOG = LogFactory.getLog(FactorTrees.class);
	private static FactorTrees application = null;
	
	private static boolean dictionaryOK = false;
	private static Map<String, String> dictionary = new HashMap<String, String>();
    
    private int semaphore = 0;
    
    private JPanel statusPanel = new JPanel();
    private JLabel idleLabel;
    private JLabel busyLabel;
    
	static {
		JDialog.setDefaultLookAndFeelDecorated(true);
		JFrame.setDefaultLookAndFeelDecorated(true);
		
		Locale locale = Locale.getDefault();
		LOG.info("Current locale: " + locale.getLanguage());
		SAXReader sax = new SAXReader();
		try {
			Document doc = sax.read(FactorTrees.class.getResourceAsStream("dictionary.xml"));
			Iterator langs = doc.getRootElement().element("supported-languages").elementIterator("language");
			String language = "czech";
			while(langs.hasNext()) {
				Element lang = (Element) langs.next();
				String langId = lang.attributeValue("id");
				if(langId.equalsIgnoreCase(locale.getLanguage())) {
					language = lang.attributeValue("section");
					LOG.info("Selected language for id '" + langId + "': " + language);
					break;
				}
			}
			
			Iterator items = doc.getRootElement().element(language).elementIterator("item");
			while(items.hasNext()) {
				Element item = (Element) items.next();
				dictionary.put(item.attributeValue("key"), item.attributeValue("value"));
			}
			dictionaryOK = true;
		} catch (DocumentException e) {
			dictionaryOK = false;
			e.printStackTrace();
		}
	}
	
	public static String getMessage(String key) {
		return dictionary.get(key);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if(!dictionaryOK) {
			LOG.fatal("Can not initialize application. Translations 'dictionary.xml' is broken");
		} else {
			FactorTrees.getInstance();
		}
	}
	
	public FactorTrees() {
		super(getMessage("AppTitle"));
		setSize(800, 700);
		setIconImage(LookNFeel.appIcon.getImage());
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		statusPanel.setBorder(LookNFeel.normalBorder);
		idleLabel = new JLabel(getMessage("ApplicationIsIdle"), LookNFeel.idleIcon, JLabel.RIGHT);
		idleLabel.setForeground(Color.DARK_GRAY);
		statusPanel.add(idleLabel);
		busyLabel = new JLabel(getMessage("ApplicationIsBusy"), LookNFeel.busyIcon, JLabel.RIGHT);
		busyLabel.setForeground(Color.RED);
		
		add(new MenuBar(), BorderLayout.NORTH);
		add(new PromotionPanel(), BorderLayout.CENTER);
		add(statusPanel, BorderLayout.SOUTH);
		
		setVisible(true);
	}
	
	public static FactorTrees getInstance() {
        if(application == null) {
            application = new FactorTrees();
        }
		return application;
	}
    
    public synchronized void incSemaphore() {
        semaphore++;
        LOG.debug("Locking semaphore: " + semaphore);
    	statusPanel.remove(idleLabel);
    	statusPanel.add(busyLabel);
    	busyLabel.revalidate();
    	busyLabel.repaint();
    }
    
    public synchronized boolean releaseSemaphore() {
        semaphore--;
        LOG.debug("Releasing semaphore: " + semaphore);
        if(semaphore == 0) {
        	statusPanel.remove(busyLabel);
        	statusPanel.add(idleLabel);
        	idleLabel.revalidate();
        	idleLabel.repaint();
        	return true;
        } else {
        	return false;
        }
    }
}
