/* 
 * File:	RightSpecialFactorsDialogue.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.FactorTrees;
import eu.glorya.math.diplomka.model.ModelFactory;
import eu.glorya.math.diplomka.model.RightSpecialFactorTree;
import eu.glorya.math.diplomka.model.SearchFinishedListener;
import eu.glorya.math.diplomka.model.type.SpecialFactorTree;

public class RightSpecialFactorsDialogue extends JDialog {
    private static final long serialVersionUID = -5347769509466696454L;

    private static final Log LOG = LogFactory.getLog(RightSpecialFactorsDialogue.class);

    private JTextField factorLength;

    public RightSpecialFactorsDialogue(Frame owner) {
        super(owner, FactorTrees.getMessage("menu.FindRightSpecialFactors"));
        setModal(true);
        
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 2));
        
        panel.add(new JLabel(FactorTrees.getMessage("factors.EnterLength")));

        factorLength = new JTextField("100", 10);
        panel.add(factorLength);
        
        JButton ok = new JButton(FactorTrees.getMessage("button.Search"));
        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
            	try {
	                int fl = Integer.parseInt(factorLength.getText());
	                if(fl < 1) {
	                	throw new NumberFormatException();
	                }
	                LOG.debug("Looking for right special factors of length: " + fl);
	                
	                FactorTrees.getInstance().incSemaphore();
	                Map<String, JMenuItem> menu = MenuBar.getInstance().getRunMenu();
	                menu.get("WordConfig").setEnabled(false);
	                
	                new RightSpecialFactorTree(ModelFactory.getWord().getWord(),
	                		ModelFactory.getWordConfig().getMorphism())
	                			.buildTree(fl, new SearchLsnr());
            	} catch(NumberFormatException e) {
                	JOptionPane.showMessageDialog(RightSpecialFactorsDialogue.this, FactorTrees.getMessage("error.InvalidInput"), FactorTrees.getMessage("exception.Error"), JOptionPane.ERROR_MESSAGE);
            	} catch(Exception e) {
                	JOptionPane.showMessageDialog(RightSpecialFactorsDialogue.this, FactorTrees.getMessage("error.UnexpectedError") + e.getMessage(), FactorTrees.getMessage("exception.Error"), JOptionPane.ERROR_MESSAGE);
            	} finally {
                    RightSpecialFactorsDialogue.this.dispose();
            	}
            }
        });
        panel.add(ok);
        
        JButton cancel = new JButton(FactorTrees.getMessage("button.Cancel"));
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RightSpecialFactorsDialogue.this.dispose();
            }
        });
        panel.add(cancel);
        
        add(panel, BorderLayout.CENTER);
        Dimension d = panel.getPreferredSize();
        d.height += 30;
        d.width = d.width < 450 ? 450 : d.width;
        setSize(d);
        setResizable(false);
        setVisible(true);
    }

    private class SearchLsnr implements SearchFinishedListener {
        public void searchFinished(SpecialFactorTree source) {
            RightSpecialFactorTree RSTree = (RightSpecialFactorTree) source; 
            new TreeWindow(FactorTrees.getMessage("right.specialFactor.tree"), RSTree.getRoot(), TreeWindow.RIGHT_FACTOR_TREE, source.getWord(), source.getRules());
            if(FactorTrees.getInstance().releaseSemaphore()) {
                Map<String, JMenuItem> menu = MenuBar.getInstance().getRunMenu();
                menu.get("WordConfig").setEnabled(true);
            }
        }
    }
}
