/* 
 * File:	FactorInfoWindow.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.gui.util;

import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JWindow;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.gui.util.TreeView.Node;
import eu.glorya.math.diplomka.model.FactorHelper;
import eu.glorya.math.diplomka.model.FactorHelper.OccurenceInfo;

public class FactorInfoWindow extends JWindow {
    private static final long serialVersionUID = 6821638406155633676L;
    private static final Log LOG = LogFactory.getLog(FactorInfoWindow.class);
    
    private String factor;
    private JTextArea info;
    
    private boolean closeSemaphore = false;
    private int closeCounter = 10;
    private TimerTask closeTask;
    private Timer timer = new Timer();

    public FactorInfoWindow(Window parent, Node factorNode, char[] word, Map<Character, String> rules) {
        super(parent);
        
        this.factor = factorNode.getFactor();
        
        closeTask = new TimerTask() {
			@Override
			public void run() {
				synchronized (closeTask) {
					if(!closeSemaphore) {
						if(closeCounter == 0) {
							LOG.debug("Closing factor info");
							FactorInfoWindow.this.dispose();
							timer.cancel();
						} else {
							closeCounter--;
						}
					}
				}
			}
		};
        
        java.awt.event.MouseListener closeProcessor = new MouseAdapter() {
        	private int counter = 0;
        	
        	private void show() {
				synchronized (closeTask) {
					closeSemaphore = true;
					closeCounter = 3;
					counter++;
				}
        	}
        	private void hide() {
				synchronized (closeTask) {
					counter--;
					if(counter == 0)
						closeSemaphore = false;
				}
        	}
			@Override
			public void mouseReleased(MouseEvent e) {
				hide();
			}
			@Override
			public void mousePressed(MouseEvent e) {
				show();
			}
			@Override
			public void mouseExited(MouseEvent e) {
				hide();
			}
			@Override
			public void mouseEntered(MouseEvent e) {
				show();
			}
		};
        
        FactorHelper helper = new FactorHelper(this.factor, word, rules);
        List<OccurenceInfo> occurecies = helper.getOccurecies(50);
        LOG.info("Factor '" + factor + "' occuracies: " + occurecies);

        String infoText = factor + "\n";
        infoText += "|w| = " + factor.length() + "\n";
        infoText += "µ = " + factorNode.possibleExtensions + "\n";
        for(OccurenceInfo o : occurecies) {
            infoText += o.prefix + " " + o.index + " " + o.suffix + " ... ";
            infoText += o.decomposition.prefix + " φ(" + o.decomposition.template + ") " + o.decomposition.suffix + "\n";
        }
        
        info = new JTextArea(infoText);
        info.setEditable(false);
        info.setFocusable(true);
        info.addMouseListener(closeProcessor);
        
        JScrollPane scrollPane = new JScrollPane(info);
        scrollPane.addMouseListener(closeProcessor);
        scrollPane.getVerticalScrollBar().addMouseListener(closeProcessor);
        scrollPane.getHorizontalScrollBar().addMouseListener(closeProcessor);
        add(scrollPane);
        
        setSize(300, 200);
        
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowOpened(WindowEvent e) {
                LOG.debug("Factor info window opened");
                info.scrollRectToVisible(new Rectangle(0, 0, 1, 1));
                timer.schedule(closeTask, 200, 200);
            }
        });
    }
}
