/* 
 * File:	MouseListener.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.gui.util;

import java.awt.Point;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.gui.util.TreeView.Node;

public class MouseListener extends MouseAdapter {
    private static final Log LOG = LogFactory.getLog(MouseListener.class);
    
    private Node treeRoot;
    private int x;
    private int y;
    private int nodeWidth;
    private int nodeHeight;
    
    private Window window;
    private char[] word;
    private Map<Character, String> rules;
    
    public MouseListener(Window window, Node treeRoot, int nodeWidth, int nodeHeight, char[] word, Map<Character, String> rules) {
        this.treeRoot = treeRoot;
        this.nodeWidth = nodeWidth;
        this.nodeHeight = nodeHeight;
        this.word = word;
        this.rules = rules;
        this.window = window;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        if(LOG.isDebugEnabled())
            LOG.debug("Mouse clicked at: " + e.getX() + " - " + e.getY());
        this.x = e.getX();
        this.y = e.getY();
        Node foundNode = recurseSearch(treeRoot);
        if(LOG.isDebugEnabled())
            LOG.debug("Clicked node: " + foundNode);
        if(foundNode != null && foundNode.parent != null) {
            Point p = e.getComponent().getLocationOnScreen();
            FactorInfoWindow w = new FactorInfoWindow(window, foundNode, word, rules);
            w.setLocation(p.x + foundNode.x + nodeWidth, p.y + foundNode.y);
            w.setVisible(true);
        }
    }
    
    private Node recurseSearch(Node node) {
        if(node.x < x && node.x + nodeWidth > x && node.y < y && node.y + nodeHeight > y)
            return node;
        else if(node.children == null)
            return null;
        else
            for(Node child : node.children) {
                Node result = recurseSearch(child);
                if(result != null)
                    return result;
            }
        return null;
    }
}
