/* 
 * File:	TreeView.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.gui.util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Window;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.gui.TreeWindow;
import eu.glorya.math.diplomka.model.search.State;

public class TreeView extends JComponent {
	private static final long serialVersionUID = -8055916264215807010L;
	private static final Log LOG = LogFactory.getLog(TreeView.class);
    
    private int treeType = TreeWindow.UNDEFINED_FACTOR_TREE;

	private static Font font = new Font("Monospaced", Font.PLAIN, 14);
	private int charHeight;
	private int charWidth;
    
    private int charBaseLine;
    private int charShift;
	private Dimension dimension = null;

	private Window window;
	private char[] word;
	private Map<Character, String> rules;
	
	private Node treeRoot;
	private int levels = 0;
	private int treeWidth = 0;
//	private List<Node> leafs = new ArrayList<Node>();
    
    private int nodeEWidth;
    private int nodeWidth;
    private int nodeHeight;

	public class Node {
        protected String letter;
        protected int possibleExtensions = 0;
        protected Node parent = null;
		protected List<Node> children = null;
        
        protected int x = -10000;
        protected int y = -10000;
		
        protected float logicalPosition = 0;
        protected boolean matchesWord = false;
//        private int level;

		public Node(Node parent, State node) {
			possibleExtensions = node.getPossibleExtensions();
            if(node.getLetter() == null) {
                this.letter = "ε";
            } else {
                this.letter = "" + node.getLetter();
            }
			if(parent != null) {
				this.parent = parent;
				parent.addChild(this);
			}
		}

		public void markBranchEnd(int level) {
//			this.level = level;
//			boolean added = false;
//			for(int i=0, c=leafs.size(); i<c; i++) {
//				Node l = leafs.get(i);
//				if(l.level < level) {
//					logicalPosition = l.logicalPosition;
//					for(int j=i, d=leafs.size(); j<d; j++) {
//						leafs.get(j).logicalPosition++;
//					}
//					added = true;
//					leafs.add(i, this);
//					break;
//				}
//			}
//			if(!added) {
//				logicalPosition = treeWidth;
//				leafs.add(this);
//			}
//			if(LOG.isDebugEnabled()) {
//				LOG.debug("Added level: " + level);
//				for(Node n : leafs) {
//					LOG.debug("Leaf: " + n);
//				}
//			}

			logicalPosition = treeWidth;
			checkWord(level);
		}
		
		private boolean checkWord(int level) {
			if(parent == null) {
				return true;
			}
			if(word[level-2] == letter.charAt(0) && parent.checkWord(level - 1))
				matchesWord = true;
			return matchesWord;
		}
		
		private void addChild(Node child) {
			if(children == null) {
				children = new ArrayList<Node>();
			}
			children.add(child);
		}
		
		public void adjustTree() {
			logicalPosition = getPosition();
		}

		private float getPosition() {
			if(children != null) {
				float pos = 0;
				for(Node child : children)
					pos += child.getPosition();
				logicalPosition = pos / children.size();
			}
			return logicalPosition;
		}

        @Override
        public String toString() {
            return "Factor: " + getFactor();
        }
        
        protected String getFactor() {
            StringBuilder strBuilder = new StringBuilder(letter);
            Node p = parent;
            while(p != null && p.parent != null) {
                strBuilder.append(p.letter);
                p = p.parent;
            }
            if(treeType == TreeWindow.RIGHT_FACTOR_TREE)
                return strBuilder.toString();
            else
                return strBuilder.reverse().toString();
        }
	}

	public TreeView(Window window, State treeRoot, int type, char[] word, Map<Character, String> rules) {
		super();
        this.treeType = type;
        this.word = word;
        this.rules = rules;
        this.window = window;
        
		setBorder(new CompoundBorder(new CompoundBorder(new EmptyBorder(5, 50, 5, 5), new LineBorder(Color.WHITE, 2)), new EmptyBorder(10, 10, 10, 10)));
		setFont(font);

		this.treeRoot = new Node(null, treeRoot);

		levels = 0;
		treeWidth = 0;
		copyTree(this.treeRoot, treeRoot, 1);
		if(LOG.isDebugEnabled()) {
			LOG.debug("Tree levels: " + levels);
			LOG.debug("Tree width: " + treeWidth);
		}
		this.treeRoot.adjustTree();
	}

	private void copyTree(Node parent, State node, int level) {
		if(level > levels)
			levels = level;
        Collection<State> childs = node.getNext();
        if(childs.isEmpty()) {
            parent.markBranchEnd(level);
            treeWidth++;
        } else {
            int widthChange = -1;
            for(State c : childs) {
                Node child = new Node(parent, c);
                copyTree(child, c, level + 1);
                widthChange++;
            }
		}
	}

	@Override
	public Dimension getPreferredSize() {
		if(dimension == null) {
			FontMetrics fm = getGraphics().getFontMetrics();
			charHeight = fm.getHeight();
			charWidth = fm.charWidth('m');
			charBaseLine = fm.getLeading() + fm.getAscent();
            nodeEWidth = charWidth + 32;
            charShift = 3;
            nodeWidth = charWidth + 4;
            nodeHeight = charHeight + 2;
            Insets insets = getInsets();
			dimension = new Dimension(
                    (treeWidth * nodeEWidth) + insets.left + insets.right,
                    (levels * nodeHeight) + insets.top + insets.bottom);
			if(LOG.isDebugEnabled())
				LOG.debug("Computed component dimension: " + dimension);
            
            addMouseListener(new MouseListener(window, this.treeRoot, nodeWidth, nodeHeight, word, rules));
		}
		return dimension;
	}

	@Override
	protected void paintComponent(Graphics g) {
        Insets insets = getInsets();
		Rectangle clip = new Rectangle(getVisibleRect());
//        LOG.debug(clip);
        g.setColor(Color.BLUE);
        g.fillRect(clip.x, clip.y, clip.width, clip.height);
		int startY = ((int) (clip.y / nodeHeight) - 1) * nodeHeight;
		int endY = (((int) (clip.height / nodeHeight) + 1) * nodeHeight) + startY;
        int startX = insets.left;
//        int endX = clip.width + startX;
		
		List<Node> currentLevel = new ArrayList<Node>();
		currentLevel.add(treeRoot);
		int drawY = insets.top;
		int halfNodeX = nodeWidth / 2;
        int halfNodeY = nodeHeight / 2;
		boolean end;
		do {
			end = true;
			List<Node> nextLevel = new ArrayList<Node>();
			int muSum = 0;
			for(Node curL : currentLevel) {
				muSum += (curL.possibleExtensions - 1);
                int x = (int)(curL.logicalPosition * nodeEWidth) + startX + 15;
                if(curL.children != null) {
                    g.setColor(Color.WHITE);
                    end = false;
                    for(Node child : curL.children) {
                        nextLevel.add(child);
                        g.drawLine(x + halfNodeX, drawY + halfNodeY, (int)(child.logicalPosition * nodeEWidth) + startX + 15 + halfNodeX, drawY + nodeHeight);
                    }
                }
				if(drawY >= startY) {
					if(curL.matchesWord)
						g.setColor(Color.YELLOW);
					else
						g.setColor(Color.WHITE);
                    g.fillRect(x, drawY, nodeWidth, nodeHeight);
                    g.setColor(Color.GRAY);
                    g.drawRect(x, drawY, nodeWidth, nodeHeight);
                    g.setColor(Color.BLACK);
					g.drawString(curL.letter, x + charShift, drawY + charBaseLine);
                    curL.x = x;
                    curL.y = drawY;
				} else {
				    curL.x = -10000;
                }
			}
			if(drawY >= startY && muSum >= 0) {
                g.setColor(Color.WHITE);
				g.drawString("" + muSum, 2, drawY + charBaseLine);
			}
			currentLevel = nextLevel;
			drawY += nodeHeight;
			if(drawY > endY)
				end = true;
		} while(!end);
	}
}
