/* 
 * File:	FactorHelper.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.model;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.model.search.FactorSearchAutomat;

public class FactorHelper {
    private static final Log LOG = LogFactory.getLog(FactorHelper.class);
    
    public class OccurenceInfo {
        public char prefix = 'ε';
        public char suffix = 'ε';
        public int index;
        public FactorDecomposition decomposition;
        
        @Override
        public String toString() {
            return "(" + prefix + ")" + index + "(" + suffix + ")=" + decomposition;
        }
    }
    
    public class FactorDecomposition {
        public String prefix = "ε";
        public String template = "ε";
        public String suffix = "ε";
        
        public FactorDecomposition inner = null;
        
        @Override
        public String toString() {
            return prefix + "fi(" + template + ")" + suffix;
        }
    }
    
    private String factor;
    private FactorSearchAutomat automat;
    private char[] word;
    private Map<Character, String> rules;

    public FactorHelper(String factor, char[] word, Map<Character, String> rules) {
        automat = new FactorSearchAutomat(factor);
        this.factor = factor;
        this.word = word;
        this.rules = rules;
    }
    
    public List<OccurenceInfo> getOccurecies(int limit) {
        List<OccurenceInfo> list = new ArrayList<OccurenceInfo>();
        automat.start();
        int index = 0;
        int factorL = factor.length();
        LOG.debug("Factor length: " + factorL + ", word: " + word.length);
        for(char ch : word) {
            if(automat.search(ch)) {
                OccurenceInfo info = new OccurenceInfo();
                info.index = index-factorL+1;
                if(index+1 < word.length)
                    info.suffix = word[index+1];
                if(index-factorL >= 0)
                    info.prefix = word[index-factorL];
                info.decomposition = getDecomposition(factorL, info);
                list.add(info);
                if(list.size() == limit) {
                    LOG.info("Limit reached, interrupting search");
                    break;
                }
            }
            index++;
        }
        return list;
    }
    
    public FactorDecomposition getDecomposition(int factorL, OccurenceInfo оccurence) {
        int begin = оccurence.index;
        int end = factorL + оccurence.index - 1;
        int currentIndex = 0;
        int letterIndex = 0;
        
        while(currentIndex < begin) {
            currentIndex += rules.get(word[letterIndex]).length();
            letterIndex++;
        }
        
        FactorDecomposition dec = new FactorDecomposition();
        
        if(currentIndex > end) {
        	dec.prefix = new String(word, begin, factorL);
        	return dec;
        }
        
        if(begin < currentIndex) {
            StringBuilder prefix = new StringBuilder();
            for(int i = begin; i < currentIndex; i++) {
                prefix.append(word[i]);
            }
            dec.prefix = prefix.toString();
        }
        
        // currentIndex points to the first letter inside the factor which could be generated
        
        StringBuilder template = new StringBuilder();
        while(currentIndex + rules.get(word[letterIndex]).length() - 1 <= end) {
            template.append(word[letterIndex]);
            currentIndex += rules.get(word[letterIndex]).length();
            letterIndex++;
        }
        if(template.length() > 0)
            dec.template = template.toString();
        
        // currentIndex points outside the factor or inside the factor, but next substitution is too long
        
        if(currentIndex <= end) {
            StringBuilder suffix = new StringBuilder();
            for(int i = currentIndex; i <= end; i++) {
                suffix.append(word[i]);
            }
            dec.suffix = suffix.toString();
        }
        
        return dec;
    }
    
    public void finishDecomposition(FactorDecomposition dec) {
    	
    }
}
