/* 
 * File:	RightSpecialFactorTree.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.model;

import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.model.search.FactorSearchAutomat;
import eu.glorya.math.diplomka.model.search.State;
import eu.glorya.math.diplomka.model.type.SpecialFactorTree;

public class RightSpecialFactorTree implements Runnable, SpecialFactorTree {
    private static final Log LOG = LogFactory.getLog(RightSpecialFactorTree.class);
    
    private State root;
    private int levels = 1;
    private SearchFinishedListener lsnr;
    private char[] word;
    private Map<Character, String> rules;
    
    public RightSpecialFactorTree(char[] word, Map<Character, String> rules) {
        root = new State(null);
        this.word = word;
        this.rules = rules;
    }
    
    public State getRoot() {
        return root;
    }
    
    public void buildTree(int levels, SearchFinishedListener lsnr) {
        this.levels = levels;
        this.lsnr = lsnr;
        new Thread(this).start();
    }
    
    public void run() {
        try {
            long buildTime = System.currentTimeMillis();
            FactorSearchAutomat search = new FactorSearchAutomat(root);
            int length = word.length;
            long start = 0;
            long time = 0;
            double avgTime = 0;
            for(int i=0; i<levels; i++) {
                start = System.currentTimeMillis();
                search.start();
                for(int idx = length - 1; idx > 0; idx--) {
                    if(search.search(word[idx]) && idx <= length - 1 - i)
                        search.processCurrent(word[idx+i], word[idx-1]);
                }
                search.phaseThree();
                time = System.currentTimeMillis() - start;
                avgTime += time;
                if(LOG.isInfoEnabled())
                    LOG.info("Right special factors of length " + (i+1) + " found in " + time + "ms");
            }
            avgTime /= levels;
            LOG.info("Right special factor tree of " + levels + " built in " + (System.currentTimeMillis() - buildTime) + "ms, average time: " + avgTime);
        } catch(Exception e) {
            LOG.error("Exception detected: " + e.getMessage());
        } finally {
            lsnr.searchFinished(this);
        }
    }

	public char[] getWord() {
		return word;
	}

	public Map<Character, String> getRules() {
		return rules;
	}
}
