package eu.glorya.math.diplomka.model;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.glorya.math.diplomka.FactorTrees;
import eu.glorya.math.diplomka.model.exception.WordConfigException;

public class WordConfig {
    private static final Log LOG = LogFactory.getLog(WordConfig.class);
    
//    private String rulesString = "0-001,1-2,2-01";
//    private String rulesString = "0-001,1-00";
//    private String rulesString = "0-012,1-125,2-234,3-435,4-5,5-0";
    private String rulesString = "0-001,1-02,2-1"; // zadani 1
//    private String rulesString = "0-001,1-00"; // zadani 2
    private Character startLetter = '0';
    private int wordLength = 3000000;
	private Map<Character, String> morphism;
//    private Set<Character> alphabet = new HashSet<Character>();
	
	public Character getStartLetter() {
		return startLetter;
	}

	public void setStartLetter(Character startLetter) throws WordConfigException {
        if(LOG.isDebugEnabled())
            LOG.debug("Setting entered start letter: " + startLetter);
        this.startLetter = startLetter;
        if(!morphism.containsKey(startLetter))
            throw new WordConfigException(FactorTrees.getMessage("exception.InvalidStartLetterNotInMorpism") + morphism.keySet());
        if(!morphism.get(startLetter).startsWith(startLetter.toString())) {
        	throw new WordConfigException(FactorTrees.getMessage("exception.InvalidStartLetter") + startLetter);
        }
	}

	public Map<Character, String> getMorphism() {
		return morphism;
	}

	public void addRule(Character letter, String substitution) throws WordConfigException {
		if(morphism.containsKey(letter)) {
			throw new WordConfigException(FactorTrees.getMessage("exception.MorphismAlreadyContainsTheLetter"));
		}
		morphism.put(letter, substitution);
	}
    
    public String getRulesString() {
        return rulesString;
    }

    public void setRulesString(String rulesString) throws WordConfigException {
        if(LOG.isDebugEnabled())
            LOG.debug("Setting entered rules: " + rulesString);
        this.rulesString = rulesString;
        
        morphism = new HashMap<Character, String>();
//        alphabet.clear();
        
        StringTokenizer tokenizer = new StringTokenizer(rulesString, ",");
        while(tokenizer.hasMoreTokens()) {
            String rule = tokenizer.nextToken().trim();
            if(LOG.isDebugEnabled())
                LOG.debug("Processing rule: " + rule);
            StringTokenizer ruleTokenizer = new StringTokenizer(rule, "-");
            String from = ruleTokenizer.nextToken();
            if(!ruleTokenizer.hasMoreTokens() || (from.length() != 1)) {
                throw new WordConfigException(FactorTrees.getMessage("exception.InvalidRuleFormat") + rule);
            }
            String to = ruleTokenizer.nextToken();
            if(to.length() < 1)
                throw new WordConfigException(FactorTrees.getMessage("exception.InvalidRuleFormat") + rule);
            if(LOG.isDebugEnabled())
                LOG.debug("New rule: " + from + " -> " + to);
//            alphabet.add(from.charAt(0));
//            for(char ch : to.toCharArray())
//                alphabet.add(ch);
            morphism.put(from.charAt(0), to);
        }
    }

    public Integer getWordLength() {
        return wordLength;
    }

    public void setWordLength(int wordLength) throws WordConfigException {
        if(wordLength < 2) {
            throw new WordConfigException(FactorTrees.getMessage("exception.InvalidWordLength") + wordLength);
        }
        this.wordLength = wordLength;
    }
//
//    public Set<Character> getAlphabet() {
//        return alphabet;
//    }
}
