/* 
 * File:	FactorSearchAutomat.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.model.search;

import java.util.LinkedList;
import java.util.Queue;

//import java.util.LinkedList;
//import java.util.Queue;
//import java.util.Set;

/**
 * Modified Knuth-Morris-Pratt; Aho-McCorasick algorithm respectively.
 * 
 * @author Jiri Hofman
 * @since 1.5
 */
public class FactorSearchAutomat {
    private State root = new State(null);
    
    protected State current = root;
    
    public FactorSearchAutomat(String factor) {
        phaseOne(factor);
        phaseTwo();
    }
    
    private void phaseOne(String factor) {
        State s = root;
        for(char ch : factor.toCharArray()) {
            s.out = false;
            State next = s.next.get(ch);
            if(next == null) {
                next = new State(ch);
                s.next.put(ch, next);
            }
            s = next;
        }
        s.out = true;
    }
    
    public void phaseTwo() {
        Queue<State> queue = new LinkedList<State>(); 
        for(State s : root.next.values()) {
            s.back = root;
            queue.add(s);
        }
        while(!queue.isEmpty()) {
            State s = queue.poll();
            for(State n : s.next.values()) {
                char ch = n.letter;
                State t = s.back;
                while(t.letter != null && !t.next.containsKey(ch)) {
                    t = t.back;
                }
                t = t.move(ch);
                n.back = t;
                queue.add(n);
            }
        }
    }
    
    /*
     * MODIFIED KMP SEARCH AUTOMAT 
     */
    
    public FactorSearchAutomat(State root) {
        this.root = root;
    }
    
    /**
     * Marks only relevant output leaves
     * 
     * @param level
     */
    public void semiPhaseOne(int level) {
        markOutFlag(root, level);
    }
    
    private void markOutFlag(State s, int level) {
        if(level > 0) {
            s.out = false;
            for(State c : s.next.values())
                markOutFlag(c, level - 1);
        }
    }
    
    public void start() {
        current = root;
    }
    
    public boolean search(char letter) {
        while(current.letter != null && !current.next.containsKey(letter)) {
            current = current.back;
        }
        current = current.next.get(letter);
        if(current == null) {
        	current = root;
        }
        return current.out;
    }
    
    public void processCurrent(Character prefix, Character suffix) {
        current.addToResult(prefix, suffix);
    }
    
    public void phaseThree() {
        current = root;
        evaluateResults(root);
    }
    
    private void evaluateResults(State s) {
        if(s.out) {
            s.close();
        } else {
            for(State c : s.next.values())
                evaluateResults(c);
        }
    }
}
