/* 
 * File:	State.java
 * Author:	Jiri Hofman
 *
 * (c) Copyright Jiri Hofman 2007
 */

package eu.glorya.math.diplomka.model.search;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class State {
    /**
     * null refers to Epsilon (empty prefix) - this is allowed ONLY for the
     * automat root
     */
    protected Character letter = null;
    protected Map<Character, State> next = new HashMap<Character, State>();
    protected State back = null;

    protected boolean out = true;
    /**
     * Maps a suffix letter of a found factor to a set of factor letter prefixes
     */
    private Map<Character, Set<Character>> counter = null;
    
    private int possibleExtensions = 0;

    public State(Character ch) {
        letter = ch;
        back = this;
    }

    protected State move(Character ch) {
        State newState = next.get(ch);
        return newState == null ? back : newState;
    }

    @Override
    public String toString() {
        return "" + letter;
    }

    public void addToResult(Character prefix, Character suffix) {
        if(counter == null) {
            counter = new HashMap<Character, Set<Character>>();
        }
        Set<Character> prefixes;
        if(!counter.containsKey(suffix)) {
            prefixes = new HashSet<Character>();
            counter.put(suffix, prefixes);
        } else {
            prefixes = counter.get(suffix);
        }
        prefixes.add(prefix);
    }

    public void close() {
        for(Character ch : counter.keySet()) {
        	int prolongs = counter.get(ch).size();
            if(prolongs > 1) {
                State n = new State(ch);
                State t = back;
                while(t.letter != null && !t.next.containsKey(ch)) { //null refers to automat root
                    t = t.back;
                }
                t = t.move(ch);
                n.back = t;
                n.possibleExtensions = prolongs;
                next.put(ch, n);
            }
        }
        counter = null;
        out = false;
    }

    public Character getLetter() {
        return letter;
    }

    public Collection<State> getNext() {
        return next.values();
    }

	public int getPossibleExtensions() {
		return possibleExtensions;
	}
}
